use utf8;
package {{$name}};
our $VERSION = '{{$dist->version}}';
use Moose;
use namespace::autoclean;

# ABSTRACT: {{$name}} package needs a propper abstract


__PACKAGE__->meta->make_immutable;

__END__

=head1 DESCRIPTION

=head1 SYNOPSIS

=head1 ATTRIBUTES

=head1 METHODS
